/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCChildList;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.components.LongTextField;
import net.sf.dc.console.components.PictureField;
import net.sf.dc.console.components.ShortTextField;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.console.dialogs.QuestionBox;
import net.sf.dc.console.movie.ImdbMovieSearchForm;
import net.sf.dc.console.software.AmazonSoftwareSearchForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.helpsystem.JavaHelp;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.resources.DCTooltips;
import net.sf.dc.wf.requests.CloseItemFormRequest;
import net.sf.dc.wf.requests.RemoveRowsRequest;
import net.sf.dc.wf.requests.UpdateRowRequest;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.StringUtils;
import net.sf.ext.util.Utilities;


/**
 * Form used to represent one item in the database or to create
 * a new item. The item form is very generic as it is build up by 
 * using the information present in the module. 
 * Child items are displayed using the DCChildList component 
 * 
 * @see net.sf.dc.console.components.DCChildList 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.6
 */
public class ItemForm extends DCFrame {
    
    private DCObject dco;
    private final DCModule module;

    private JLabel[] cLabels;
    private JComponent[] cInputFields;

    private int y = 0;
    
    private final JTabbedPane tabbedPane = new JTabbedPane();
    
    private final Color colorRequired = ComponentFactory.getRequiredColor();
    private final Color color = Settings.getValueAsColor(Repository.settings.stColor);
    private boolean picturesShown = false;
    private boolean childrenShown = false;
    private boolean hideFields = false;
    
    private Class clazz;
    private final boolean update;
    private final int row;
    
    private DCChildList childList;

    /**
     * Initializes and shows the item form 
     * 
     * @param readonly specifies if data can be edited / changed (and saved)
     * @param updateMode Update / Insert mode
     * @param dco Data Crow Object to be displayed
     * @param imdb use imdb update facilities
     * @param amazon use amazon update facilitie
     * @param clazz the clazz which triggered this item to be opened
     * @param row the UI row number
     */
    public ItemForm(    JFrame parent,
                        boolean readonly,
                        boolean update,
                        DCObject dco, 
                        boolean imdb,
                        boolean amazon,
                        Class clazz,
                        int row) {
        super(parent);
        
        JavaHelp.setHelpIndex("dc.ItemForm");
        
        this.dco = dco;
        this.clazz = clazz;
        this.row = row;             
        this.update = update;   
        this.module = dco.getModule();
        this.hideFields = !Settings.getValueAsBoolean(Repository.settings.stShowHiddenFieldsInForm);
        
        setResizable(true);

        setIconImage(IconLibrary._icoMain.getImage());
        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        
        if (!update) {
            setTitle(Resources.getText(DCLabels.lblNewItem, module.getObjectName()));
        } else {
            String id = (String) dco.getValue(DCObject._ID);
            setTitle(Resources.getText(DCLabels.lblItem, dco.getDisplayString(1) ));
        }
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        this.getContentPane().setBackground(color);
        this.setBackground(color);
        this.getContentPane().setLayout(Layout.getGBL());
        
        initializeComponents();

        JPanel panelActions = getActionPanel(readonly, amazon, imdb);
        
        addInputPanel();
        addChildrenPanel();        
        addPictureTabs(); 
        
        getContentPane().add(tabbedPane,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(0,0,15,0), 0, 0));        
        getContentPane().add(panelActions,  Layout.getGBC(0, 1, 1, 1, 0.0, 0.0
                            ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));        

        setRequiredFields();
        setReadonly(readonly);
        setData(dco.getValues(), true);        
        
        pack();
        
        setSize(Settings.getValueAsDimension(getItemSizeSetting()));
            
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
        setVisible(true);
    }
    
    /**
     * Closes this form by setting the visibility to false
     */
    public void close() {
        Settings.setValue(getItemSizeSetting(), getSize());
        
        if (cInputFields != null) {
            for (int i = 0; i < cInputFields.length; i++) {
                
                if (cInputFields[i] instanceof PictureField) {
                	((PictureField) cInputFields[i]).flush();
                }
                cInputFields[i] = null;
            }
    
            for (int i = 0; i < cLabels.length; i++) {
                cLabels[i] = null;
            }
            
            clazz = null;
            dco = null;
            cLabels = null;
            cInputFields = null;
            
            if (childList != null) {
                childList.flush();
                childList = null;
            }
        }
        
        dispose();
    }  
    
    /**
     * Applies data to this form
     * @param values    values to be displayed. Values is based on all fields, both
     *                  hidden and visible
     * @param overwrite indicates if existing values should be overwritten
     * @param destinationFields the field indices which are targeted to be filled
     *                  with the values
     */
    public void setData(Object[] values, boolean overwrite) {
        int[] fields = dco.getFieldIndices();
        for (int i = 0; i < dco.getFieldIndices().length; i++) {
            Object obj = ComponentFactory.getComponentValue(
                    dco.getFieldType(fields[i]),
                    cInputFields[fields[i]]);
            
            boolean empty = obj == null;
            if (!empty) {
                empty = StringUtils.isEmpty(obj.toString());
            }
            
            if ((empty || overwrite) && values[i] != "") {
                ComponentFactory.setComponentValue(
                        dco.getFieldType(fields[i]),
                        cInputFields[fields[i]],
                        values[i]);
            }
        }
    }    
    
    /**
     * Sets all the fields in this form to readonly
     * @param readonly
     */
    private void setReadonly(boolean readonly) {
        if (readonly) {
            for (int i = 0; i < cInputFields.length; i++) {
                cInputFields[i].setEnabled(false);
                cInputFields[i].setBackground(ComponentFactory.getDisabledColor());
            }
        }        
    }
    
    /**
     * Marks fields as being required. The label is set to the required field 
     * color
     */
    private void setRequiredFields() {
        int[] required = dco.getRequiredFields(); 
        for (int i = 0; i < dco.getRequiredFields().length; i++) {
            cLabels[required[i]].setForeground(colorRequired);
        }  
    }    
    
    /**
     * Retrieves the size settings key for the current active module
     */
    private String getItemSizeSetting() {
        String setting = Repository.settings.stItemFormViewSize;
        if (module.getIndex() == DCModules._MOVIE) {
            setting = Repository.settings.stMovieItemFormViewSize;
        } else if (module.getIndex() == DCModules._SOFTWARE) {
            setting = Repository.settings.stSoftwareItemFormViewSize;
        } else if (module.getIndex() == DCModules._SOFTWARECAT) {
            setting = Repository.settings.stSoftwareCatItemFormViewSize;
        } else if (module.getIndex() == DCModules._MUSIC) {
            setting = Repository.settings.stMusicItemFormViewSize;
        } 
        return setting;
    }
    
    /**
     * Returns the data from screen
     */
    private Object[] getData() {
        Object[] values = dco.getValues();
        for (int i = 0; i < cInputFields.length; i++) {
            Object value = ComponentFactory.getComponentValue(
                            dco.getFieldType(i), 
                            cInputFields[i]);
            values[i] = value;
        }
        
        if (!update) {
            values[DCObject._ID] = "";
        }
        return values;
    }

    /**
     * Starts a Amazon update. The update will show its own dialog
     * and transfers data selected by the user back to this item
     * form.
     */
    private void startAmazonUpdate() {
        Object[] data = this.getData();
        DCObject o = module.getDCObject();
        o.setValues(getData());
        new AmazonSoftwareSearchForm(o, this);
    }

    /**
     * Starts a IMDB update. The update will show its own dialog
     * and transfers data selected by the user back to this item
     * form.
     */
    private void startIMDBUpdate() {
        Object[] data = this.getData();
        DCObject o = module.getDCObject();
        o.setValues(getData());
        new ImdbMovieSearchForm(o, this);
    }

    /**
     * Deletes this item from the database
     */
    private void deleteValues() {
        new QuestionBox(Resources.getText(DCMessages.msgDeleteQuestion), this);
        
        if (QuestionBox.result.booleanValue()) {
            String id = dco.getID();
            dco.clearValues();
            dco.setValue(DCObject._ID, id);
            dco.setSilence(true);

            int tab = update ? MainFrame._SEARCHTAB : MainFrame._INSERTTAB;
            dco.addRequest(new CloseItemFormRequest(this));
            dco.addRequest(new RemoveRowsRequest(module.getIndex(),
                           tab, new String[] {id}));
            dco.delete();
        }
    }

    /**
     * Inserts or updates the Data Crow object
     */
    private void saveValues() {
        Object[] newVals = getData();
        for (int i = 0; i < newVals.length; i++) {
            if (newVals[i] == null) {
                newVals[i] = "";
            }
        }        
        
        dco.removeChildren();
        DCObject[] children = childList.getChildren();
        for (int i = 0; i < children.length; i++) {
            dco.addChild(children[i]);
        }
        
        dco.setSilence(true);
        if (!update)  {
            dco.undoChanges();
            
            for (int i = 0; i < newVals.length; i++) {
                if (!newVals[i].equals("")) {
                    dco.setValue(i, newVals[i]);
                }
            }        
            dco.setValue(DCObject._ID, "");
            
            try {
                dco.addRequest(new CloseItemFormRequest(this));
                dco.setSilence(true);
                dco.saveNew();
            } catch (ValidationException vExp) {
                // when an error is encountered, saving is stopped
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
                return;
            }
            
        } else {
            // get the orginal values
            dco.setValues(newVals);
            int tab = update ? MainFrame._SEARCHTAB : MainFrame._INSERTTAB;
            dco.addRequest(new UpdateRowRequest(tab, dco, true, true));
            try {
                dco.addRequest(new CloseItemFormRequest(this));
                dco.setSilence(true);
                dco.saveUpdate();
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
                return;
            }
        }
    }

    private void initializeComponents() {
        String[] labelText = dco.getLabels();
        int[] maxFieldLength = dco.getMaxFieldLengths();
        
        int[] componentType = dco.getFieldTypes();
        cLabels = new JLabel[componentType.length];
        cInputFields = new JComponent[componentType.length];
        
        // The ID Field (readonly)
        cLabels[0] = ComponentFactory.getLabel(labelText[0]);
        ShortTextField textField = ComponentFactory.getShortTextField(100);
        textField.setBackground(ComponentFactory.getDisabledColor());
        textField.setEditable(false);
        
        cInputFields[0] = textField;
        for (int i = 1; i < componentType.length; i ++) {
            cLabels[i] = ComponentFactory.getLabel(labelText[i]);
            JComponent comp = ComponentFactory.getComponent(componentType[i], 
                                                            labelText[i],
                                                            maxFieldLength[i]);
            cInputFields[i]= comp;
        }
    }
    
    private void addInputPanel() {
        JPanel panelInfo = new JPanel(); 
        JPanel panelTech = new JPanel();
        panelInfo.setLayout(Layout.getGBL());
        panelInfo.setBackground(color);
        panelTech.setLayout(Layout.getGBL());
        panelTech.setBackground(color);
        
        int[] componentType = dco.getFieldTypes();
      
        int yTech = 0;
        int yInfo = 0;
        boolean technicalInfoPresent = false;
        for (int j = 0 ; j < componentType.length; j++) {
        	if (((hideFields && !dco.isHidden(j)) || !hideFields) &&
                !(cInputFields[j] instanceof PictureField) && 
                j != dco.getParentReferenceFieldIndex()) {
                
                JComponent component = cInputFields[j];
                cLabels[j].setBackground(color);
                
                int stretch = GridBagConstraints.HORIZONTAL;
                int factor = 1;
                
                if (componentType[j] == ComponentFactory._LONGTEXTFIELD) {
                    stretch = GridBagConstraints.BOTH; 
                    factor  = 50;
    
                    LongTextField longText = (LongTextField) component;
                    longText.setMargin(new Insets(1, 1, 1, 5));

                    JScrollPane pane = new JScrollPane(longText);
                    pane.setBorder(ComponentFactory.getBorder());
                    pane.setPreferredSize(new Dimension(100,100));                    
                    pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                    pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
                    component = pane;
                } else {
                	component.setPreferredSize(new Dimension(200, 20));
                }
                cLabels[j].setPreferredSize(new Dimension(100, 20));
                
                if (!dco.getField(j).isTechnicalInfo()) {
                    panelInfo.add(cLabels[j], Layout.getGBC(0, yInfo, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                              new Insets(0, 0, 0, 5), 0, 0));
                    panelInfo.add(component,  Layout.getGBC(1, yInfo, 1, 1, factor, factor
                             ,GridBagConstraints.NORTHWEST, stretch,
                              new Insets(0, 0, 0, 0), 0, 0));
                    yInfo++;
                } else {
                    technicalInfoPresent = true;
                    panelTech.add(cLabels[j], Layout.getGBC(0, yTech, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(0, 0, 0, 5), 0, 0));
                    panelTech.add(component,  Layout.getGBC(1, yTech, 1, 1, factor, factor
                            ,GridBagConstraints.NORTHWEST, stretch,
                             new Insets(0, 0, 0, 0), 0, 0));
                    yTech++;
                }
                
            }
        }
        tabbedPane.addTab(Resources.getText(DCLabels.lblDescription), IconLibrary._icoDescription, panelInfo);
        if (technicalInfoPresent) {
            JPanel dummy = new JPanel();
            dummy.setLayout(Layout.getGBL());
            dummy.setBackground(color);
            dummy.add(panelTech,  Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets(2, 0, 0, 0), 0, 0));
            tabbedPane.addTab(Resources.getText(DCLabels.lblTechnicalInfo), IconLibrary._icoTechInfo, dummy);
        }
    }
    

    private void addChildrenPanel() {
        Collection children = dco.getChildren();
        if (children.size() > 0) {
            childList = new DCChildList(module.getChildModule(), children);
            childrenShown = true;
        } else {
            childList = new DCChildList(module.getChildModule(), null);
        }
        
        if (module.getChildModule() != null) {
            tabbedPane.addTab(module.getChildModule().getObjectName() + "s", childList);
        }
    }
    
    private void addPictureTabs() {
        int[] componentType = dco.getFieldTypes();
        
        for (int j = 0 ; j < componentType.length; j++) {
            JComponent component = cInputFields[j];
            if (((hideFields && !dco.isHidden(j)) || !hideFields) && 
                (cInputFields[j] instanceof PictureField)) {
                
                JPanel panel = new JPanel(); 
                panel.setLayout(Layout.getGBL());
                panel.setBackground(color);
                
                component.setPreferredSize(component.getMinimumSize());
                panel.add(component, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                          new Insets(5, 5, 5, 5), 0, 0));
                
                tabbedPane.addTab(dco.getField(j).getLabel(), 
                                  IconLibrary._icoPicture,
                                  panel);
            }
        }
    }
    
    private JPanel getActionPanel(boolean readonly, boolean amazon, boolean imdb) {
        JPanel panel = new JPanel(false);
        panel.setBackground(color);
        panel.setLayout(Layout.getGBL());
        
        String name = module.getObjectName();
        JButton buttonClose = ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
        JButton buttonDelete = ComponentFactory.getButton(Resources.getText(DCLabels.lblDelete, name));
        JButton buttonInternet = null;
        
        if (imdb) {
            buttonInternet = ComponentFactory.getButton(Resources.getText(DCLabels.lblIMDBUpdate));
            buttonInternet.addActionListener(new IMDBAction());
            buttonInternet.setToolTipText(Resources.getText(DCTooltips.tpIMDBUpdate));
            buttonInternet.setMnemonic(KeyEvent.VK_U);
        } else if (amazon) {
            buttonInternet = ComponentFactory.getButton(
                             Resources.getText(DCLabels.lblAmazonUpdate));
            buttonInternet.addActionListener(new AmazonAction());
            buttonInternet.setToolTipText(Resources.getText(DCTooltips.tpAmazonUpdate));
            buttonInternet.setMnemonic(KeyEvent.VK_U);
        }        

        JButton buttonSave = ComponentFactory.getButton(Resources.getText(DCLabels.lblSave, name));
        buttonSave.setMnemonic(KeyEvent.VK_S);
        buttonSave.setToolTipText(Resources.getText(DCTooltips.tpSave));
        buttonSave.addActionListener(new SaveAction());
        
        buttonClose.setMnemonic(KeyEvent.VK_C);
        buttonClose.setToolTipText(Resources.getText(DCTooltips.tpClose));
        buttonClose.addActionListener(new ExitAction());
        
        buttonDelete.setMnemonic(KeyEvent.VK_D);
        buttonDelete.setToolTipText(Resources.getText(DCTooltips.tpDelete));
        buttonDelete.addActionListener(new DeleteAction());
        
        if (!readonly) {
        panel.add(buttonSave, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                  new Insets(5, 5, 5, 5), 0, 0));
        }
                        
        if (amazon || imdb) {
            panel.add(buttonInternet, Layout.getGBC(1, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                      new Insets(5, 5, 5, 5), 0, 0));
        }
        
        if (update && !readonly) {
            panel.add(buttonDelete, Layout.getGBC(2, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                      new Insets(5, 5, 5, 5), 0, 0));
        }
        
        panel.add(buttonClose , Layout.getGBC(3, 0, 1, 1, 1.0, 1.0
                 ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                  new Insets(5, 5, 5, 5), 0, 0));
        
        return panel;
    }

    private class IMDBAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startIMDBUpdate();
        }
    }

    private class AmazonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startAmazonUpdate();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveValues();
        }
    }

    private class DeleteAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            deleteValues();
        }
    }

    private class ExitAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
