/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.SearchForm;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.console.musicfile.InsertMusicFilePanel;
import net.sf.dc.console.musicfile.SearchMusicFilePanel;
import net.sf.dc.console.musicfile.organizer.MusicFileOrganizerDialog;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportMusicFilesCD;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;

/**
 * The Music module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class MusicFileModule extends DCModule {
    
    private final DCMenuBar menuBar;
    private final MusicFileOrganizerDialog musicOrganizer = new MusicFileOrganizerDialog(DataCrow.mainFrame);
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame,
                           Resources.getText(DCLabels.lblMusicFileImport));
    
    /**
     * Initializes this module
     */
    public MusicFileModule() {
        super(DCModules._MUSIC,
              Resources.getText(DCLabels.lblModuleMusicFile),
              "musicfiles",
              "mufi",
              "musicfiles",
              false,
              false);
        menuBar = new DCMenuBar(this);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoMusicModule;
    }
    
    public DCMenuBar getMenuBar() {
        return menuBar;
    }    
    
    public String getObjectName() {
        return Resources.getText(DCLabels.lblMusicFile);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(Resources.getText(DCLabels.lblTools));
        
        JMenuItem menuImport = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuImport16,
                                                Resources.getText(DCLabels.lblMusicFileImport));
        menuImport.addActionListener(new ImportCDAction());

        menu.add(menuImport);
        
        return menu;
    }    
    
    /**
     * Return the object managed by this module 
     * @see MusicFile
     */    
    public DCObject getDCObject() {
        return new MusicFile();
    }
        
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertMusicFilePanel();
        searchPanel = new SearchMusicFilePanel();
        searchForm = new SearchForm(this, searchPanel);
    }
        
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsMusic;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stMusicRequiredFields;
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._MUSIC));
        importCDDialog.setVisible(true);
    }
    
    /**
     * Returns the music CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     * 
     * @return
     */     
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                        	  boolean confirmEachDir,
											  String nameOfCD) {
        return new ImportMusicFilesCD(ui, sourceDir, confirmEachDir, nameOfCD);
    }
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DCFieldDefinition[14];
        fields[MusicFile._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");
        fields[MusicFile._A_FILENAME]	= 	new DCFieldDefinition(Resources.getText(DCLabels.lblFileName),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");
        fields[MusicFile._B_TITLE] = 	    new DCFieldDefinition(Resources.getText(DCLabels.lblTitle),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Title");
        fields[MusicFile._C_ARTIST] 	 =  new DCFieldDefinition(Resources.getText(DCLabels.lblArtist),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Artist");                                       
        fields[MusicFile._D_ALBUM] = 	new DCFieldDefinition(Resources.getText(DCLabels.lblAlbum),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Album");                                     
        fields[MusicFile._E_YEAR] 	      = new DCFieldDefinition(Resources.getText(DCLabels.lblYear),
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Year");   
        fields[MusicFile._F_GENRE] 	      = new DCFieldDefinition(Resources.getText(DCLabels.lblGenre),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Genre");     
        fields[MusicFile._G_PLAYLENGTH]   = new DCFieldDefinition(Resources.getText(DCLabels.lblPlayLength),
                                        false,
                                        false,
                                        false,
                                        20,
                                        ComponentFactory._TIMEFIELD,
                                        Repository.datatypes._STRING,
                                        "Length");    
        fields[MusicFile._H_QUALITY] 	  = new DCFieldDefinition(Resources.getText(DCLabels.lblQuality),
                                        false,
                                        true,
                                        true,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Quality");
        fields[MusicFile._I_ENCODING] 	  = new DCFieldDefinition(Resources.getText(DCLabels.lblEncoding),
                						false,
                                        true,
                                        true,
										255,
										ComponentFactory._SHORTTEXTFIELD,
									    Repository.datatypes._STRING,
                						"Encoding");
        fields[MusicFile._J_LYRIC] 	  	  = new DCFieldDefinition(Resources.getText(DCLabels.lblLyric),
										false,
                                        true,
                                        false,
										8000,
										ComponentFactory._LONGTEXTFIELD,
									    Repository.datatypes._STRING,
										"Lyric");
        fields[MusicFile._K_FILESIZE]      = new DCFieldDefinition(Resources.getText(DCLabels.lblFilesize),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Filesize");           
        fields[MusicFile._L_LOCATION]      = new DCFieldDefinition(Resources.getText(DCLabels.lblLocation),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location");   
        fields[MusicFile._M_RATING] = new DCFieldDefinition(Resources.getText(DCLabels.lblRating),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");           
    }

    public DCObject[] getDefaultData() {
        return new DCObject[0];
    }
    
    private class MusicOrganizerAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            musicOrganizer.setVisible(true);
        }
    }    
    
    private class ImportCDAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }    
}
