/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.wf;

import java.sql.ResultSet;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.SwingUtilities;

import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.core.objects.musicfiles.MusicFile;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.core.objects.software.SoftwareCategory;
import net.sf.dc.db.Database;
import net.sf.dc.db.Query;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.messages.Messages;
import net.sf.dc.wf.requests.RequestCollection;
import net.sf.ext.util.Base64;

/**
 * This is the layer between the UI and the database. Checks for data
 * integrity and checks on bussiness rules are done here
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class WorkFlow {

    private static Database db;

    /**
     * Initializes the database
     */
    public static void initDatabase() {
        if (db == null) {
            db = new Database();
        }
    }

    /**
     * Runs a predefined query
     * @param sQuery query statement
     * @param requestors  requests to the UI
     */
    public void runQuery(String sQuery, RequestCollection requestors) {
        initDatabase();
        db.runQuery(sQuery, requestors);
    }

    /**
     * Searches for values. The query is put in the query queue. The results are
     * past to the caller by using the RequestorCollection
     * @param o the object which is used as the condition
     * @param options query options
     */
    public void searchValues(DCObject o, QueryOptions queryOptions) {
        initDatabase();
        db.searchValues(o, queryOptions);
    }

    /**
     * Inserts new values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     * @param o the object to be inserted
     */
    public void insertValues(DCObject o) {
        initDatabase();
        db.insertValues(o);
    }

    /**
     * Updates values. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     * @param o  the object to be updated
     */
    public void updateValues(DCObject o) {
        initDatabase();
        db.updateValues(o);
    }

    /**
     * Deletes the object. The query is put in the query queue. Actions are
     * past to the caller by using the RequestorCollection
     * @param o the object to be updated
     */    
    public void deleteValues(DCObject o) {
        initDatabase();
        db.deleteValues(o);
    }
    
    public Collection runQueryDirect(String sQuery, boolean catchErrors) throws Exception{
        initDatabase();
        return db.runQueryDirect(sQuery, catchErrors);     
    }

    /**
     * Runs a query directly, bypassing the query queue
     * @param o Data Crow object
     */
    public Collection runQueryDirect(DCObject o) {
        initDatabase();
        return db.runQueryDirect(o);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Errors are returned to the caller (not thrown)
     * @param Query Query object
     */
    public Collection runQueryDirect(Query query) {
        initDatabase();
        return db.runQueryDirect(query);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * Does not close the resultset!
     * @param sQuery query statement
     */
    public ResultSet runQueryDirectUnclosed(String sQuery) throws Exception {
        initDatabase();
        return db.runQueryDirectUnclosed(sQuery, false);
    }

    /**
     * Does not put the query in a queue but executes it immediately.
     * @param sQuery query statement
     */
    public Collection runQueryDirect(String sQuery) {
        initDatabase();
        return db.runQueryDirect(sQuery);
    }

    /**
     * Passes requests thru to the UI (<url>MainFrame</url>)
     * @param vTable data to be shown
     * @param requestorObjects collection of requests
     */
    public void displayResults(Collection objects, RequestCollection requestorObjects, boolean qryWasSuccess) {
        Runnable runMe = new UIUpdater(objects, requestorObjects, qryWasSuccess);
        SwingUtilities.invokeLater(runMe);
        runMe = null;
    }
    
    

    /**
     * Checks if the object is unique
     * @param o object to be checked
     * @param bUpdate specifies if the object exists already in the
     *        database or not
     */
    public boolean checkUniqueness(DCObject o, boolean bUpdate) {
        initDatabase();
        return db.uniqueValues(o, bUpdate);
    }

    /**
     * Converts a resultset to a DataCrowObject Vector
     * @param result result set from a query
     */
    public static Collection convertToDCObjects(ResultSet result) {
        ArrayList objects = new ArrayList();

        try {
        	result.isLast();
        } catch (Exception exp) {
            return objects;
        }
        
        try {
            String sTableName = result.getMetaData().getTableName(1);
            int columns = result.getMetaData().getColumnCount();
            String[] sColumns = new String[columns];

            int k = 0;
            for (int i = 1; i < columns + 1; i++) {
                sColumns[k] = result.getMetaData().getColumnName(i);
                k++;
            }

            while (result.next()) {
                Object[] rowData = new String[columns];
                for (int j = 0; j < sColumns.length; j++) {
                    int type = result.getMetaData().getColumnType(j + 1);
                    Object value = null;
                    
                    if (type == Types.BINARY) {
                        value = result.getBytes(sColumns[j]);
                        if (value != null) {
                            value = Base64.encode((byte[]) value);
                        } 
                    } else {
                        value = result.getString(sColumns[j]);    
                    }
                    rowData[j] = value == null ? "" : value;
                }

                DCObject dcObj = null;
                if (sTableName.toUpperCase().equals("SOFTWARE")) {
                    dcObj = new Software();
                } else if (sTableName.toUpperCase().equals("SOFTWARECATEGORIES")) {
                    dcObj = new SoftwareCategory();
                } else if (sTableName.toUpperCase().equals("MOVIES")) {
                    dcObj = new Movie();
                } else if (sTableName.toUpperCase().equals("MUSICFILES")) {
                    dcObj = new MusicFile();
                } else if (sTableName.toUpperCase().equals("AUDIOCDS")) { 
                    dcObj = new AudioCD();
                } else if (sTableName.toUpperCase().equals("AUDIOTRACKS")) { 
                    dcObj = new AudioTrack();
                }
                
                dcObj.setValues(rowData);
                objects.add(dcObj);
                rowData = null;
                dcObj = null;
            }
        } catch (Exception exp) {
            Messages.add(exp);
        } 
        return objects;
    }
}
